﻿FreeRefills = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")
local L = AceLibrary("AceLocale-2.2"):new("FreeRefills")

L:RegisterTranslations("enUS", function() return {
	["Add"] = true,
	["Add an item to the refill list. (Usage: /freerefills add [Item Link] #)"] = true,
	["[Item Link] <number of items>"] = true,
	["Add Stack"] = true,
	["Add an item to the refill list by stack. (Usage: /freerefills addstack [Item Link] #)"] = true,
	["[Item Link] <number of stacks>"] = true,
	["Remove"] = true,
	["Remove an item from the refill list. (Usage: /freerefills del [Item Link])"] = true,
	["[Item Link]"] = true,
	["Overstock"] = true,
	["Toggle Overstocking.  If an item is sold in stacks and cannot meet your exact requested amount, you can opt to buy slightly more (Overstock)."] = true,
	["Process Merchant"] = true,
	["Toggle Merchant Looting."] = true,
	["Process Bank"] = true,
	["Toggle Bank Looting."] = true,
	["Clear"] = true,
	["Clear the refill list completely."] = true,
	["List"] = true,
	["List the items in the refill list."] = true,
	["Added %s - Amount:%d"] = true,
	["Removed %s."] = true,
	["Item List Cleared"] = true,
	["Refill List: "] = true,
	["     None"] = true,
	["Invalid input; input must be '<item> #'."] = true,
}
end)

L:RegisterTranslations("koKR", function() return {
	["Add"] = "추가",
	["Add an item to the refill list. (Usage: /freerefills add [Item Link] #)"] = "리필 목록에 아이템을 추가합니다. (사용법: /freerefills add [아이템링크] #)",
	["[Item Link] <number of items>"] = "[아이템 링크] <아이템 수량>",
	["Add Stack"] = "묶음 추가",
	["Add an item to the refill list by stack. (Usage: /freerefills addstack [Item Link] #)"] ="묶음량에 따라서 리필 목록에 아이템을 추가합니다. (사용법: /freerefills addstack [아이템 링크] #)",
	["[Item Link] <number of stacks>"] = "[아이템 링크] <묶음의 양>",
	["Remove"] = "제거",
	["Remove an item from the refill list. (Usage: /freerefills del [Item Link])"] = "리필 목록에서 아이템을 제거합니다. (사용법: /freerefills del [아이템 링크])",
	["[Item Link]"] = "[아이템 링크]",
	["Overstock"] = "재고",
	["Toggle Overstocking.  If an item is sold in stacks and cannot meet your exact requested amount, you can opt to buy slightly more (Overstock)."] = "재고관리 기능을 토글합니다.  만약에 아이템이 묶음으로 팔리고 정확한 요구량을 충족시키지 못할 경우에, 약간의 여유량을 가지고 아이템을 구입할 것입니다.",
	["Process Merchant"] = "상점",
	["Toggle Merchant Looting."] = "상점에서 아이템 구입 기능을 토글합니다.",
	["Process Bank"] = "은행",
	["Toggle Bank Looting."] = "은행에서 아이템 이동 기능을 토글합니다.",
	["Clear"] = "초기화",
	["Clear the refill list completely."] = "리필 목록을 완전히 초기화 합니다.",
	["List"] = "목록",
	["List the items in the refill list."] = "리필 목록을 출력합니다.",
	["Added %s - Amount:%d"] = "추가된 아이템 ID#%s - 수량:%d",
	["Removed %s."] = "제거된 아이템 %s",
	["Item List Cleared"] = "아이템 목록이 초기화 되었습니다",
	["Refill List: "] = "리필 목록: ",
	["     None"] = "     없음",
	["Invalid input; input must be '<item> #'."] = "잘못된 입력; '<아이템> #' 형식이어야 합니다",
}
end)


function FreeRefills:OnInitialize()
	local defaults = {
		overstock = false,
		merchant = true,
		bank = true,
		refills = {},
	}

	local args = {
		type = 'group',
		args = {
			add = {
				name = L["Add"], type = 'text',
			    desc = L["Add an item to the refill list. (Usage: /freerefills add [Item Link] #)"],
				get = false,
			    set = function(text) self:AddRefillItem(text) end,
				usage = L["[Item Link] <number of items>"],
				guiHidden = true
			},
			addstack = {
				name = L["Add Stack"], type = 'text',
			    desc = L["Add an item to the refill list by stack. (Usage: /freerefills addstack [Item Link] #)"],
			    get = false,
				set = function(text) self:AddRefillItemStack(text) end,
				usage = L["[Item Link] <number of stacks>"],
				guiHidden = true
			},
			del = {
				name = L["Remove"], type = 'text',
			    desc = L["Remove an item from the refill list. (Usage: /freerefills del [Item Link])"],
				get = false,
			    set = function(text) self:RemoveRefillItem(text) end,
				usage = L["[Item Link]"],
				guiHidden = true
			},
			overstock = {
				name = L["Overstock"], type = 'toggle',
				desc = L["Toggle Overstocking.  If an item is sold in stacks and cannot meet your exact requested amount, you can opt to buy slightly more (Overstock)."],
				get = function() return self.db.profile.overstock end,
				set = function() self.db.profile.overstock = not self.db.profile.overstock end
			},
			merchant = {
				name = L["Process Merchant"], type = 'toggle',
				desc = L["Toggle Merchant Looting."],
				get = function() return self.db.profile.merchant end,
				set = function()
					if self.db.profile.merchant == true then
						self:UnregisterEvent("MERCHANT_SHOW")
						self.db.profile.merchant = false
					elseif self.db.profile.merchant == false then
						self:RegisterEvent("MERCHANT_SHOW")
						self.db.profile.merchant = true
					end
				end
			},
			bank = {
				name = L["Process Bank"], type = 'toggle',
				desc = L["Toggle Bank Looting."],
				get = function() return self.db.profile.bank end,
				set = function()
					if self.db.profile.bank == true then
						self:UnregisterEvent("BANKFRAME_OPENED")
						self.db.profile.bank = false
					elseif self.db.profile.bank == false then
						self:RegisterEvent("BANKFRAME_OPENED")
						self.db.profile.bank = true
					end
				end
			},
			reset = {
				name = L["Clear"], type = 'execute',
			    desc = L["Clear the refill list completely."],
			    func = function() self:ClearRefills() end
			},
			list = {
				name = L["List"], type = 'execute',
			    desc = L["List the items in the refill list."],
			    func = function() self:List() end
			}
		}
	}
	
	self:RegisterDB("FreeRefillsDB")
	self:RegisterDefaults('profile', defaults )
	
	self:RegisterChatCommand({ "/freerefills", "/fr" }, args, "FREEREFILLS")
	
	if not self.db.char.notFirstTime then
		self.db.char = {} -- clean out the old data from my non-profile usage
		self.db.char.notFirstTime = true
		self:SetProfile('char')
	end
	
	--Copy and Nuke old Ace1 SavedData
	local realm,char = GetRealmName(), UnitName("player")
	if FreeRefillsDB[realm..':'..char] then
		for k,v in pairs(FreeRefillsDB[realm..':'..char]) do
			if k == "mood" then
				if v == "Pessimest" then self.db.profile.overstock = true
				else self.db.profile.overstock = false end
			else
				self.db.profile.refills[k] = v
			end
		end
		if FreeRefillsDB[realm..':'..char] then FreeRefillsDB[realm..':'..char] = nil end
	end
end

function FreeRefills:OnEnable()
	if self.db.profile.merchant then self:RegisterEvent("MERCHANT_SHOW") end
	if self.db.profile.bank then self:RegisterEvent("BANKFRAME_OPENED") end
end

function FreeRefills:debug(msg)
	-- self:Print("|cff0099CC"..msg.."|r")
end

--[[--------------------------------------------------------------------------------
  Merchant Processing
-----------------------------------------------------------------------------------]]

function FreeRefills:MERCHANT_SHOW()
	self:debug("Merchant Opened")
	for itemID, amount in pairs(self.db.profile.refills) do
		local merchSlot = self:ItemInStock(itemID)
		if merchSlot then
			self:debug("Item Found in Merchant Slot #"..merchSlot)
			local amountInBags = GetItemCount(itemID)
			if amountInBags >= amount then
				self:debug("Already have enough in bags.")
			else
				local stackCycle, stackRem = self:CalculateNeeded(amount, amountInBags, merchSlot)
				if stackCycle then self:debug("Need to buy "..stackCycle.." more full stacks.") end
				if stackRem then self:debug("Need to buy "..stackRem.." item(s) (Remainder stack)") end
				local numAvailable = select(5, GetMerchantItemInfo(merchSlot))
				if stackCycle then
					if stackCycle*GetMerchantItemMaxStack(merchSlot) > numAvailable and numAvailable > 0 then
						self:debug("Can only buy "..numAvailable)
						BuyMerchantItem(merchSlot, numAvailable)
					else
						for i = 1, stackCycle do
							BuyMerchantItem(merchSlot, GetMerchantItemMaxStack(merchSlot))
						end
					end
				end
				if stackRem then
					if stackRem > numAvailable and numAvailable > 0 then
						self:debug("Can only buy "..numAvailable)
						stackRem = numAvailable
					end
					BuyMerchantItem(merchSlot, stackRem)
				end
			end
		end
	end
end

function FreeRefills:ItemInStock(itemLink)
	self:debug("Searching for Item: "..itemLink)
	local itemToFind = GetItemInfo(itemLink)
	local numItems = GetMerchantNumItems()
	for i = 1, numItems do
		local merchantItemLink = GetMerchantItemLink(i)
		if merchantItemLink then
			local item = GetItemInfo(merchantItemLink)
			if item == itemToFind then
				return i
			end
		end
	end
end

function FreeRefills:CalculateNeeded(amount, amountInBags, merchSlot)
	-- more detailed logic regauding stacks should be taken care of here
	local amountNeeded
	local stackSize = select(4, GetMerchantItemInfo(merchSlot))
	local stackedAmount
	local stackedNeeded
	local maxBuy = GetMerchantItemMaxStack(merchSlot)
	local stackCycle
	local stackRem
	
	self:debug("Stacksize: "..stackSize)
	
	if stackSize ~= 1 then
		stackedAmount = amount / stackSize
		stackedNeeded = amountInBags / stackSize
		amountNeeded = stackedAmount - stackedNeeded
		self:debug("Unrounded Needed: "..amountNeeded)
		if self.db.profile.overstock == true then
			stackCycle = math.ceil(amountNeeded)
		else
			stackCycle = math.floor(amountNeeded)
		end
	else
		local finalNeeded = amount - amountInBags
		self:debug("Need to buy "..finalNeeded.." more.  Merchant sells in max stacks of "..maxBuy..".")
		if finalNeeded > maxBuy then
			local stacks = finalNeeded / maxBuy
			stackCycle = math.floor(stacks)
			stackRem = finalNeeded - (stackCycle * maxBuy)
		else
			stackRem = finalNeeded
		end
	end

	if stackCycle and stackCycle <= 0 then stackCycle = nil end
	if stackRem and stackRem <= 0 then stackRem = nil end
	
	return stackCycle, stackRem
end

--[[--------------------------------------------------------------------------------
  Bank Processing
-----------------------------------------------------------------------------------]]

function FreeRefills:BANKFRAME_OPENED()
	self:debug("Bank Opened")
	for itemID,amount in pairs(self.db.profile.refills) do
		local amountInBags = GetItemCount(itemID)
		local amountInBank = GetItemCount(itemID, true)
		if amountInBank > amountInBags then
			amountInBank = amountInBank - amountInBags
			local amountNeeded = amount - amountInBags
			if amountInBags >= amount then
				self:debug("Already have enough in bags.")
			else
				if amountNeeded > amountInBank then
					amountNeeded = amountInBank end
				self:debug(amountNeeded.." item(s) to be looted.")
				self:LootBank(itemID, amountNeeded)
			end
		end
	end
end

function FreeRefills:LootBank(itemID, amountNeeded)
	self:debug("Bank Looting Logic: #"..itemID.." - "..amountNeeded)
	--Scan the main bank first
	for slot = 1, 24 do
		local slotID = GetContainerItemLink(-1, slot)
		if slotID == itemID then
			local slotCount = select(2, GetContainerItemInfo(-1, slot))
			self:debug(slotCount.." Item(s) Found in Bank:Slot "..slot)
			if slotCount <= amountNeeded then --We don't have to split this stack, we can cheat and use UseContainerItem
				self:debug("Looting Entire Stack")
				UseContainerItem(-1, slot)
				amountNeeded = amountNeeded - slotCount
				self:debug(slotCount.." Looted - Need "..amountNeeded.." more item(s)")
			else
				SplitContainerItem(-1, slot, amountNeeded)
				for bag = 4, 0, -1 do
					self:debug("Scanning Bag #"..bag)
					local size = GetContainerNumSlots(bag);
					if (size > 0) then
						for slot = 1, size, 1 do
							local slotID = GetContainerItemLink(bag,slot)
							if not slotID then
								if bag == 0 then
									PutItemInBackpack()
								else
									local bagID
									local bag = tonumber(bag)
									bagID = bag + 19
									PutItemInBag(bagID)
								end
							end
						end
					end
				end
				local amountLooted = amountNeeded
				amountNeeded = amountNeeded - amountLooted -- This should always result in 0
				self:debug(amountLooted.." Looted - Need "..amountNeeded.." more item(s)")
			end
		end
		if amountNeeded == 0 then self:debug(amountNeeded.." item(s) # "..itemID.." needed.  Next item.") return end
	end
	self:debug("Main Bank didn't cover it, scanning additional Bags")
	--Now Scan bank bags
	for bag = 5, 10, 1 do
		local size = GetContainerNumSlots(bag);
		if (size > 0) then
			for slot = 1, size do
				local slotID = GetContainerItemLink(bag,slot)
				if slotID == itemID then
					local slotCount = select(2, GetContainerItemInfo(bag, slot))
					self:debug(slotCount.." Item(s) Found in Bag "..bag..":Slot "..slot..". Need to loot "..amountNeeded)
					if slotCount <= amountNeeded then --We don't have to split this stack, we can cheat and use UseContainerItem
						self:debug("Looting Entire Stack")
						UseContainerItem(bag, slot)
						amountNeeded = amountNeeded - slotCount
						self:debug(slotCount.." Looted - Need "..amountNeeded.." more item(s)")
					else
						SplitContainerItem(bag, slot, amountNeeded)
						for bag = 4, 0, -1 do
							local size = GetContainerNumSlots(bag);
							if (size > 0) then
								for slot = 1, size, 1 do
									local slotID = GetContainerItemLink(bag,slot)
									if (not slotID) then
										if bag == 0 then
											PutItemInBackpack()
										else
											local bagID
											local bag = tonumber(bag)
											bagID = bag + 19
											PutItemInBag(bagID)
										end
									end
								end
							end
						end
						local amountLooted = amountNeeded
						amountNeeded = amountNeeded - amountLooted -- This should always result in 0
						self:debug(amountLooted.." Looted - Need "..amountNeeded.." more item(s)")
					end
				end
				if amountNeeded == 0 then self:debug(amountNeeded.." item(s) # "..itemID.." needed.  Next item.") return end
			end
		end
	end
end

--[[--------------------------------------------------------------------------------
  Shared Functions
-----------------------------------------------------------------------------------]]

--[[--------------------------------------------------------------------------------
  Command Handlers
-----------------------------------------------------------------------------------]]

function FreeRefills:AddRefillItem(msg)
	--add an item to the refill list
	local link, num = select(3, msg:find("(.+)%s(%d+)"))
	if not link or not num then
		self:Print(L["Invalid input; input must be '<item> #'."])
		return
	end
	num = tonumber(num)
	self.db.profile.refills[link] = num
	self:Print(string.format(L["Added %s - Amount:%d"], link, num))
--	self:Print("Added "..itemID.." - Amount:"..num)
end

local function GetStackSize(item)
	local itemStack = select(8, GetItemInfo(item))
	return itemStack
end

function FreeRefills:AddRefillItemStack(msg)
	--add an item to the refill list by stack
	local link, num = select(3, msg:find("(.+)%s(%d+)"))
	if not link or not num then
		self:Print(L["Invalid input; input must be '<item> #'."])
		return
	end
	num = tonumber(num)
	num = num * GetStackSize(link)
	self.db.profile.refills[link] = num
	self:Print(string.format(L["Added %s - Amount:%d"], link, num))
----	self:Print("Added "..itemID.." - Amount:"..num)
end

function FreeRefills:RemoveRefillItem(msg)
	--remove an item from the refill list
	if not self.db.profile.refills[msg] then return end
	self.db.profile.refills[msg] = nil
	self:Print(string.format(L["Removed %s."], msg))
end

function FreeRefills:ClearRefills()
	--clear the refill list
	self.db.profile.refills = {}
	self:Print(L["Item List Cleared"])
end

function FreeRefills:List()
	--report back which items are currently on the refill list, and their quanitity
	self:Print(L["Refill List: "])
	local notempty
	for itemID,amount in pairs(self.db.profile.refills) do
		self:Print("     "..itemID.." ("..amount..")")
		notempty = 1
	end
	if not notempty then self:Print(L["     None"]) end
end

